import "Turbine";
import "Turbine.Gameplay";

print = Turbine.Shell.WriteLine;
Settings = class();

function tprint (tbl, indent)
  if not indent then indent = 0 end
  for k, v in pairs(tbl) do
    formatting = string.rep("  ", indent) .. k .. ": "
    if type(v) == "table" then
      print(formatting)
      tprint(v, indent+1)
    else
      print(formatting .. v)
    end
  end
end


currentVersion = 111;

function Settings:Constructor()
	self:Load();
end

function Settings:Load()
	self.settingsTable = Turbine.PluginData.Load(Turbine.DataScope.Server,"ChallengeTracker2");
	if(self.settingsTable == nil or (self.settingsTable.version == nil or self.settingsTable.version < currentVersion)) then
		self.settingsTable = {};
		self.settingsTable.version = currentVersion;
	end

	if(self.settingsTable.players == nil) then
		self.settingsTable.players = {};
	end
	
	if(self.settingsTable.instances == nil) then
		self.settingsTable.instances = {};
		self:LoadInstances();
	end
	
	local curPlay = Turbine.Gameplay.LocalPlayer:GetInstance();
	local name = curPlay:GetName();
	if(string.sub(name,0,0) == "~") then
		print("Plugin does not work for instanced characters...crashing");
		exit();
	end
	if(curPlay:GetAlignment() ~= 1) then
		print("Plugin does not work for monster players...crashing");
		exit();
	end 
	self:AddPlayer(name);
	self.curPlayer = name;
	
	self:PerformResets();
	self:Save();
end

function Settings:LoadInstances()
	self:AddInstance("The Tower of Orthanc: Fire/Frost",{},"raid",{"Thursday","Sunday"},6);
	self:AddInstance("The Tower of Orthanc: Lightning",{},"raid",{"Thursday"},6);
	self:AddInstance("The Tower of Orthanc: Acid",{},"raid",{"Thursday"},6);
	self:AddInstance("The Tower of Orthanc: Shadow",{},"raid",{"Thursday"},6);
	self:AddInstance("The Tower of Orthanc: Saruman",{},"raid",{"Thursday"},6);
	self:AddInstance("Roots of Fangorn",{"Instance","Roots","Fangorn"},"fellowship",{"Thursday","Monday"},3);
	self:AddInstance("The Foundry",{"Challenge","Ring","orge Falls"},"fellowship",{"Thursday","Monday"},3);
	self:AddInstance("Dargn\195\161kh Unleased",{"Challenge","Darg","Unleashed"},"fellowship",{"Thursday","Monday"},3);
	self:AddInstance("Pits of Isengard",{"Challenge","Pits","Isengard"},"fellowship",{"Thursday","Monday"},3);
	self:AddInstance("Fangorn's Edge",{"Challenge","Fangorn","s Savior"},"fellowship",{"Thursday","Monday"},3);
	self:AddInstance("Draigoch's Lair",{},"raid",{"Thursday"},6);
	self:AddInstance("Helegrod: Thorog",{"Helegrod Drugoth","Raid"},"skirmish",{"Thursday"},6);
	self:AddInstance("Helegrod: Drake Wing",{"Helegrod Drake","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Helegrod: Giant Wing",{"Helegrod Giant","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Helegrod: Spider Wing",{"Helegrod Spider","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Ford of Bruinen",{"Ford","Bruinen","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Siege of Gondamon",{"Siege","Gondamon","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Trouble in Tuckborough",{"Trouble","Tuckborough","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Stand at Amon S\195\186l",{"Stand","Amon","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Strike Against Dannenglor",{"Strike Against Dannenglor","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Protectors of Thang\195\186lhad",{"Protectors","Thang","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Breaching the Necromancer's Gate",{"Breaching","Necromancer","s Gate","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Assault on the Ringwraiths' Lair",{"Assault","Ringwraiths","Lair","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("The Battle in the Tower",{"The Battle","Tower","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Survival: Barrow-Downs",{"Survival","Barrow","owns","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Defence of the Prancing Pony",{"Defence","Prancing Pony","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Thievery and Mischief",{"Thievery","Mischief","Raid"},"skirmish",{"Thursday"},6);--
	self:AddInstance("Rescue in N\195\186rz Gh\195\161shu",{"Rescue","rz Gh","shu","Raid"},"skirmish",{"Thursday"},6);
	self:AddInstance("Battle of the Deep-way",{"Battle","Deep","ay","Raid"},"skirmish",{"Thursday"},6);
	self:AddInstance("Battle of the Way of Smiths",{"Battle","Way of Smiths","Raid"},"skirmish",{"Thursday"},6);
	self:AddInstance("Battle of the Twenty-first Hall",{"Battle","Twenty","irst Hall","Raid"},"skirmish",{"Thursday"},6);
	self:AddInstance("The Icy Crevasse",{"Icy","Crevasse","Raid"},"skirmish",{"Thursday"},6);
	self:AddInstance("Attack At Dawn",{"Attack","Dawn","Raid"},"skirmish",{"Thursday"},6);
	self:AddInstance("Storm on Methedras",{"Storm","Methedras","Raid"},"skirmish",{"Thursday"},6);
	self:AddInstance("Wold - Urush",{"Warband","Urush"},"warband",{"Monday","Tuesday","Wednesday","Thursday","Friday","Saturday","Sunday"},3);
	self:AddInstance("Wold - Cinder",{"Warband","Cinder"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Wold - Hanrun",{"Warband","Hanrun"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Wold - Bughrakh",{"Warband","Bughrakh"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Norcrofts - Dah\195\161mab ",{"Warband","Dah","mab"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Norcrofts - Sk\195\186tog",{"Warband","Sk","tog"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Norcrofts - Swertr\195\173per",{"Warband","Swertr","per"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Norcrofts - Haglob",{"Warband","Haglob"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Entwash Vale - Bensengan",{"Warband","Bensengan"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Entwash Vale - Fearrhorn",{"Warband","Fearrhorn"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Entwash Vale - Mirz",{"Warband","Mirz"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Entwash Vale - D\195\162l",{"Warband","D\195\162l"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Entwash Vale - M\195\162thum",{"Warband","thum"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Entwash Vale - Urgai",{"Warband","Urgai"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Sutcrofts - Gundul",{"Warband","Gundul"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Sutcrofts - Kramp",{"Warband","Kramp"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
	self:AddInstance("Sutcrofts - Bugud",{"Warband","Bugud"},"warband",{"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},3);
end

function Settings:PerformResets()
	local curDate2 = Turbine.Engine.GetDate();
	local curDate = {};
	curDate.Year = curDate2.Year;
	curDate.DayOfYear = curDate2.DayOfYear;
	curDate.DayOfWeek = curDate2.DayOfWeek;
	curDate.Hour = curDate2.Hour;
	curDate.Minute = curDate2.Minute;
	curDate.Second = curDate2.Second;
	
	local resetDone = false;
	for k,instance in pairs(self.settingsTable.instances) do
		local resetDays = instance.resetDays;
		local resetTime = instance.resetTime;
		for q,player in pairs(self.settingsTable.players) do
			local completion = instance.completion[player];
			if(completion ~= nil and self:NeedsReset(resetDays,resetTime,completion,curDate)) then
				resetDone = true;
				instance.completion[player] = nil;
			end
		end
	end
	if(resetDone) then
		self:Save();
	end
end

function Settings:NeedsReset(resetDays,resetTime,completion,curDate)
	for k,resetDay in pairs(resetDays) do
		if(self:NeedsResetTest(resetDay,resetTime,completion,curDate)) then
			return true;
		end
	end
	return false;
end

function Settings:NeedsResetTest(resetDay,resetTime,completion,curDate)
	local resetTimeFormatted = {};
	resetTimeFormatted.DayOfWeek = resetDay;
	resetTimeFormatted.Hour = resetTime;
	resetTimeFormatted.Minute = 0;
	resetTimeFormatted.Second = 0;
	
	local comYear = completion.Year;
	local curYear = curDate.Year;

	local comDay = completion.DayOfYear;
	local curDay = curDate.DayOfYear;
	
	if(curYear > comYear + 1) then
		return true;
	elseif(curYear > comYear) then
		curDay = curDay + 365;
		if(comYear % 4 == 0) then
			curDay = curDay + 1;
		end
	end
	
	if((curDay - comDay > 7) or ((curDay - comDay == 7) and self:BeforeStrict(comDate,curDate))) then
		return true;
	end
	
	if((self:BeforeStrict(completion,resetTimeFormatted) and self:BeforeStrict(curDate,resetTimeFormatted)) or 
	   (self:BeforeStrict(resetTimeFormatted,completion) and self:BeforeStrict(resetTimeFormatted,curDate))) then
   
		if(self:BeforeStrict(curDate,completion)) then
			return true;
		else
			return false;
		end
	else

		if(self:BeforeStrict(completion,curDate)) then
			return true;
		else
			return false;
		end
	end
end

function Settings:BeforeStrict(t1,t2)
	local l1 = self:Before(t1,t2);
	local l2 = self:Before(t2,t1);
	if(l1 and (not l2)) then return true; end
	return false;
end

function Settings:Before(t1,t2)
	if(t1.DayOfWeek < t2.DayOfWeek) then
		return true;
	elseif(t2.DayOfWeek < t1.DayOfWeek) then
		return false;
	elseif(t1.Hour < t2.Hour) then
		return true;
	elseif(t2.Hour < t1.Hour) then
		return false;
	elseif(t1.Minute < t2.Minute) then
		return true;
	elseif(t2.Minute < t1.Minute) then
		return false;
	elseif(t1.Second < t2.Second) then
		return true;
	else
		return false;
	end
end

dayMapper = {};
dayMapper["Sunday"] = 1;
dayMapper["Monday"] = 2;
dayMapper["Tuesday"] = 3;
dayMapper["Wednesday"] = 4;
dayMapper["Thursday"] = 5;
dayMapper["Friday"] = 6;
dayMapper["Saturday"] = 7;
dayMapper[1] = 1;
dayMapper[2] = 2;
dayMapper[3] = 3;
dayMapper[4] = 4;
dayMapper[5] = 5;
dayMapper[6] = 6;
dayMapper[7] = 7;

function Settings:AddInstance(name,messageParts,instanceType,resetDays,resetTime)
	local instance = {};
	instance.name = name;
	instance.instanceType = instanceType;
	instance.messageParts = messageParts;
	instance.resetDays = self:ConvertDays(resetDays);
	instance.resetTime = resetTime;
	instance.completion = {};	
	instance.priority = {};
	for k,player in pairs(self.settingsTable.players) do
		instance.completion[player] = nil;
		instance.priority[player] = "standard";
	end
		
	self.settingsTable.instances[instance.name] = instance;
end

function Settings:RemoveInstance(name)
	self.settingsTable.instances[name] = nil;
end

function Settings:ConvertDays(resetDays)
	local ret = {};
	local i = 1;
	for k,day in pairs(resetDays) do
		ret[i] = dayMapper[day];
		i = i + 1;
	end
	return ret;
end

function Settings:AddPlayer(player)
	for k,playerI in pairs(self.settingsTable.players) do
		if(playerI == player) then
			return;
		end
	end
	
	self.settingsTable.players[player] = player;
	
	for k,instance in pairs(self.settingsTable.instances) do
		instance.completion[player] = nil;
		instance.priority[player] = "standard";
	end
end

function Settings:Save()
	Turbine.PluginData.Save(Turbine.DataScope.Server, "ChallengeTracker2", self.settingsTable);
end

function Settings:AddCompletion(player,instance)
	Turbine.Shell.WriteLine("Adding lock for "..instance.name.." for "..player);
	
	local curDate2 = Turbine.Engine.GetDate();
	local curDate = {};
	curDate.Year = curDate2.Year;
	curDate.DayOfYear = curDate2.DayOfYear;
	curDate.DayOfWeek = curDate2.DayOfWeek;
	curDate.Hour = curDate2.Hour;
	curDate.Minute = curDate2.Minute;
	curDate.Second = curDate2.Second;
	
	instance.completion[player] = curDate;
	self:Save();
end

function Settings:RemoveCompletion(player,instance)
	Turbine.Shell.WriteLine("Removing lock for "..instance.name.." for "..player);
	instance.completion[player] = nil;
	self:Save();
end

function Settings:GetClosestTime(resetDays,resetTime)
	local curTime = Turbine.Engine.GetDate();
	local closestTime = nil;
	
	for k,resetDay in pairs(resetDays) do
		local resetTimeFormatted = {};
		resetTimeFormatted.DayOfWeek = resetDay;
		resetTimeFormatted.Hour = resetTime;
		resetTimeFormatted.Minute = 0;
		resetTimeFormatted.Second = 0;

		if(self:BeforeStrict(curTime,resetTimeFormatted) and (closestTime == nil or self:BeforeStrict(resetTimeFormatted,closestTime))) then
			closestTime = resetTimeFormatted;
		end
	end
	
	if(closestTime ~= nil) then
		return self:TimeToString(closestTime,curTime);
	end
	
	for k,resetDay in pairs(resetDays) do
		local resetTimeFormatted = {};
		resetTimeFormatted.DayOfWeek = resetDay;
		resetTimeFormatted.Hour = resetTime;
		resetTimeFormatted.Minute = 0;
		resetTimeFormatted.Second = 0;
		
		if(closestTime == nil or (self:BeforeStrict(resetTimeFormatted,closestTime))) then
			closestTime = resetTimeFormatted;
		end
	end


	curTime.DayOfWeek = curTime.DayOfWeek - 7;
	return self:TimeToString(closestTime,curTime);
end

function Settings:TimeToString(t1,t2)
	local day = t1.DayOfWeek - t2.DayOfWeek;
	local hour = t1.Hour - t2.Hour;
	local minute = t1.Minute - t2.Minute;
	local second = t1.Second - t2.Second;
	
	if(second < 0) then
		second = second + 60;
		minute = minute - 1;
	end
	if(minute < 0) then
		minute = minute + 60;
		hour = hour -1;
	end
	if(hour < 0) then
		hour = hour + 24;
		day = day - 1;
	end
	
	return day.."D:"..hour.."H:"..minute.."M:"..second.."S";
end

function Settings:ParseMessageParts(text)
	local cur=0;
	local parts = {};
	local i = 1;
	while(true) do
		local loc = string.find(text,",",cur);
		if(loc == nil) then
			break;
		end
		parts[i] = string.sub(text,cur,loc-1);
		cur = loc + 1;
		i = i + 1;
	end
	parts[i] = string.sub(text,cur);
	return parts;
end

function Settings:CreateKeywordsText(messageParts)
	if(table.getn(messageParts) == 0) then
		return "";
	end

	local skip = true;
	for k,str in pairs(messageParts) do
		if(skip) then 
			skip = false;
			ret = str;
		else
			ret = ret..","..str;
		end
	end
	return ret;
end

function Settings:ParseResetDays(monday,tuesday,wednesday,thursday,friday,saturday,sunday)
	local ret = {};
	local i = 1;
	if(monday) then ret[i] = 2; i = i + 1; end
	if(tuesday) then ret[i] = 3; i = i + 1; end
	if(wednesday) then ret[i] = 4; i = i + 1; end
	if(thursday) then ret[i] = 5; i = i + 1; end
	if(friday) then ret[i] = 6; i = i + 1; end
	if(saturday) then ret[i] = 7; i = i + 1; end
	if(sunday) then ret[i] = 1; i = i + 1; end
	return ret;
end

function Settings:ResetTimesOnDay(resetDays,dayArg)
	for k,day in pairs(resetDays) do
		if(day == dayArg) then
			return true;
		end
	end
	return false;
end

function Settings:Update(message)
	for k,instance in pairs(self.settingsTable.instances) do
		local messageParts = instance.messageParts;
		if(table.getn(messageParts) > 0) then
			local found = true;
			for q,messagePart in pairs(messageParts) do
				if(string.find(message,messagePart) == nil) then
					found=false;
					break;
				end
			end
			if(found==true) then
				self:AddCompletion(self.curPlayer,instance);
				self:Save();
				return true;
			end
		end
	end
	return false;
end