import "Turbine.UI";
import "Turbine.UI.Lotro";
import "RambleOnion.ChallengeTracker2.Settings";

TrackerWindow = class(Turbine.UI.Window);
CTTreeNodeItem = class(Turbine.UI.TreeNode);

function CTTreeNodeItem:setAvailable()
	self.available = true;
	self:SetBackgroundColor();
	self.textbox2:SetText("Available");
end

function CTTreeNodeItem:setUnavailable(timeLeft)
	self.available = false;
	self:SetBackgroundColor();
	self.textbox2:SetText(timeLeft);
end

function CTTreeNodeItem:setSelected()
	self.selected = true;
	self:SetBackgroundColor();
end

function CTTreeNodeItem:setUnselected()
	self.selected = false;
	self:SetBackgroundColor();
end

function CTTreeNodeItem:markAsImportant()
	self.strength = 1.0;
	self:SetBackgroundColor();
end

function CTTreeNodeItem:markAsStandard()
	self.strength = 0.5;
	self:SetBackgroundColor();
end

function CTTreeNodeItem:markAsIgnore()
	self.strength = 0.25;
	self:SetBackgroundColor();
end

function CTTreeNodeItem:SetBackgroundColor()
	if(self.selected) then
		self.textbox:SetBackColor(Turbine.UI.Color(self.strength,self.strength,0.0));
	else
		self.textbox:SetBackColor(Turbine.UI.Color(0.0,0.0,self.strength));
	end
	
	
	if(self.available) then
		self.textbox2:SetBackColor(Turbine.UI.Color(0.0,self.strength,0.0));
	else
		self.textbox2:SetBackColor(Turbine.UI.Color(self.strength,0.0,0.0));
	end
end

function CTTreeNodeItem:Constructor(text)
	Turbine.UI.TreeNode.Constructor(self);
	self:SetSize(450,25);
	self.content = text;
	
	self.strength = .5;
	self.available = true;
	self.selected = false;
	
	local selfholder = self;
	
	self.textbox = Turbine.UI.Lotro.TextBox();
	self.textbox:SetPosition(0,0);
	self.textbox:SetSize(300,25);
	self.textbox:SetParent(self);
	self.textbox:SetBackColor(Turbine.UI.Color(0.0,0.0,1.0));
	self.textbox:SetSelectable(false);
	self.textbox:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
	self.textbox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.textbox:SetText(text);
	self.textbox:SetReadOnly(true);
	self.textbox.MouseClick = 
		function(sender,args) 
			if(globalWindow.currentlySelected ~=nil) then
				globalWindow:unselect();
			end
			if(globalWindow.currentlySelected ~= selfholder) then
				globalWindow:select(selfholder);
			else
				globalWindow.currentlySelected = nil;
			end
		end;
		
	self.textbox.MouseDoubleClick =
		function(sender,args)
			if(globalWindow.currentlySelected ~=nil) then
				globalWindow:unselect();
			end
			if(globalWindow.currentTab=="instances") then
				globalWindow:switchToPlayersTab(self.content);
			else
				globalWindow:switchToInstancesTab(self.content);
			end
			globalWindow.currentlySelected = nil;
		end;
	
	self.textbox2 = Turbine.UI.Lotro.TextBox();
	self.textbox2:SetPosition(300,0);
	self.textbox2:SetSize(150,25);
	self.textbox2:SetParent(self);
	self.textbox2:SetSelectable(false);
	self.textbox2:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
	self.textbox2:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.textbox2:SetReadOnly(true);
end

CTTreeNodeHeader = class(Turbine.UI.TreeNode);

function CTTreeNodeHeader:setCounts(count1,count2)
	self.count1 = count1;
	self.count2 = count2;
	if(count1>0) then
		self.textbox2:SetBackColor(Turbine.UI.Color(0.0,1.0,0.0));
		self.textbox2:SetText(count1.."("..count2..") Available");
	elseif(count2>0) then
		self.textbox2:SetBackColor(Turbine.UI.Color(0.0,.5,0.0));
		self.textbox2:SetText("("..count2..") Available");
	else
		self.textbox2:SetBackColor(Turbine.UI.Color(1.0,0.0,0.0));
		self.textbox2:SetText("Unavailable");		
	end
end

function CTTreeNodeHeader:Constructor(text)
	Turbine.UI.TreeNode.Constructor(self);
	self:SetSize(450,35);
	self.count2 = nil;
	self.count2 = nil;
	
	self.textbox = Turbine.UI.Lotro.TextBox();
	self.textbox:SetPosition(0,0);
	self.textbox:SetSize(300,35);
	self.textbox:SetParent(self);
	self.textbox:SetFont(Turbine.UI.Lotro.Font.TrajanPro18);
	self.textbox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.textbox:SetBackColor(Turbine.UI.Color(1.0,0.5,0.0));
	self.textbox:SetText(text);
	self.textbox:SetSelectable(false);
	self.textbox:SetReadOnly(true);
	
	self.textbox2 = Turbine.UI.Lotro.TextBox();
	self.textbox2:SetPosition(300,0);
	self.textbox2:SetSize(150,35);
	self.textbox2:SetParent(self);
	self.textbox2:SetSelectable(false);
	self.textbox2:SetFont(Turbine.UI.Lotro.Font.TrajanPro18);
	self.textbox2:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.textbox2:SetReadOnly(true);
end

function TrackerWindow:unselect()
	self.currentlySelected:setUnselected();
end

function TrackerWindow:select(node)
	self.currentlySelected = node;
	node:setSelected();
end

function TrackerWindow:Constructor(settings)
	Turbine.UI.Lotro.Window.Constructor(self);
	
	globalWindow = self;
	self.settings = settings;
	self:SetSize(500,500);
	self:SetPosition(0,0);
	self:SetText("Challenge Tracker");
	self:SetVisible(true);
	
	self.contextTextBox = Turbine.UI.TextBox();
	self.contextTextBox:SetEnabled(false);
	self.contextTextBox:SetPosition(91,50);
	self.contextTextBox:SetSize(317,20);
	self.contextTextBox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	self.contextTextBox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	self.contextTextBox:SetParent(self);
	
	self.mainWindow = Turbine.UI.Window();
	self.mainWindow:SetParent(self);
	self.mainWindow:SetPosition(0,70);
	self.mainWindow:SetSize(500,355);
	self.mainWindow:SetVisible(true);
	
	self.markButton = Turbine.UI.Lotro.Button();
	self.markButton:SetPosition(25,440);
	self.markButton:SetSize(130,20);
	self.markButton:SetText("Mark");
	self.markButton:SetParent(self);
	self.markButton.Click = 
		function(sender,args)
			if(globalWindow.currentlySelected ~= nil) then
				local iname = nil;
				local pname = nil;
				if(self.currentTab == "instances") then
					iname = globalWindow.currentlySelected.content;
					pname = self.currentPlayer;
				else
					iname= self.currentInstance;
					pname = globalWindow.currentlySelected.content;
				end
				
				local instance = self.settings.settingsTable.instances[iname];
				local priority = instance.priority[pname];
				
				if(instance.completion[pname] == nil) then
					self.settings:AddCompletion(pname,instance);
					
					globalWindow.currentlySelected:setUnavailable(self.settings:GetClosestTime(instance.resetDays,instance.resetTime));
					if(self.currentTab=="instances") then
						local header = self.headerMap[instance.instanceType];
						if(priority=="important") then
							header:setCounts(header.count1-1,header.count2);
						elseif(priority=="ignore") then
							--ignore
						else
							header:setCounts(header.count1,header.count2-1);
						end
					end
				else
					self.settings:RemoveCompletion(pname,instance);
					globalWindow.currentlySelected:setAvailable();
					if(self.currentTab=="instances") then 
						local header = self.headerMap[instance.instanceType];
						if(priority=="important") then
							header:setCounts(header.count1+1,header.count2);
						elseif(priority=="ignore") then
							--ignore
						else
							header:setCounts(header.count1,header.count2+1);
						end
					end
				end
			end
		end;

	self.promoteButton = Turbine.UI.Lotro.Button();
	self.promoteButton:SetPosition(188,440);
	self.promoteButton:SetSize(130,20);
	self.promoteButton:SetText("Promote");
	self.promoteButton:SetParent(self);
	self.promoteButton.Click = 
		function(sender,args)
			if(globalWindow.currentlySelected ~= nil) then
				local iname = nil;
				local pname = nil;
				if(self.currentTab == "instances") then
					iname = globalWindow.currentlySelected.content;
					pname = self.currentPlayer;
				else
					iname= self.currentInstance;
					pname = globalWindow.currentlySelected.content;
				end
				
				local instance = self.settings.settingsTable.instances[iname];
				local prevImportance = instance.priority[pname];
				
				if(prevImportance=="important") then
					--ignore
				elseif(prevImportance=="standard") then
					--promote to important
					instance.priority[pname] = "important";
					globalWindow.settings:Save();
					globalWindow.currentlySelected:markAsImportant();
					if(self.currentTab=="instances" and instance.completion[pname] == nil) then
						local header = self.headerMap[instance.instanceType];
						header:setCounts(header.count1+1,header.count2-1);
					end
				else
					--promote to standard
					instance.priority[pname] = "standard";
					globalWindow.settings:Save();
					globalWindow.currentlySelected:markAsStandard();
					if(self.currentTab=="instances" and instance.completion[pname] == nil) then
						local header = self.headerMap[instance.instanceType];
						header:setCounts(header.count1,header.count2+1);
					end	
				end
			end
		end;

	self.demoteButton = Turbine.UI.Lotro.Button();
	self.demoteButton:SetPosition(350,440);
	self.demoteButton:SetSize(130,20);
	self.demoteButton:SetText("Demote");
	self.demoteButton:SetParent(self);
	self.demoteButton.Click = 
		function(sender,args)
			if(globalWindow.currentlySelected ~= nil) then
				local iname = nil;
				local pname = nil;
				if(self.currentTab == "instances") then
					iname = globalWindow.currentlySelected.content;
					pname = self.currentPlayer;
				else
					iname= self.currentInstance;
					pname = globalWindow.currentlySelected.content;
				end
				
				local instance = self.settings.settingsTable.instances[iname];
				local prevImportance = instance.priority[pname];
				
				if(prevImportance=="important") then
					--demote to standard
					instance.priority[pname] = "standard";
					globalWindow.settings:Save();
					globalWindow.currentlySelected:markAsStandard();
					if(self.currentTab=="instances" and instance.completion[pname] == nil) then
						local header = self.headerMap[instance.instanceType];
						header:setCounts(header.count1-1,header.count2+1);
					end
				elseif(prevImportance=="standard") then
					--demote to ignore
					instance.priority[pname] = "ignore";
					globalWindow.settings:Save();
					globalWindow.currentlySelected:markAsIgnore();
					if(self.currentTab=="instances" and instance.completion[pname] == nil) then
						local header = self.headerMap[instance.instanceType];
						header:setCounts(header.count1,header.count2-1);
					end					
				else
					--ignore
				end
			end
		end;
		
	self.addButton = Turbine.UI.Lotro.Button();
	self.addButton:SetPosition(25,465);
	self.addButton:SetSize(130,20);
	self.addButton:SetText("Add Instance");
	self.addButton:SetParent(self);
	self.addButton.Click = 
		function(sender,args)
			if(globalWindow.currentlySelected ~= nil) then
				globalWindow:unselect();
			end
			globalWindow.currentlySelected = nil;
			
			if(globalWindow.popup ~= nil) then
				globalWindow.popup:Close();
			end
			
			globalWindow.popup = globalWindow:createPopup(nil);
		end;

	self.editButton = Turbine.UI.Lotro.Button();
	self.editButton:SetPosition(188,465);
	self.editButton:SetSize(130,20);
	self.editButton:SetText("Edit Instance");
	self.editButton:SetParent(self);
	self.editButton.Click = 
		function(sender,args)
			local iname = nil;
			if(globalWindow.currentTab == "instances") then
				if(globalWindow.currentlySelected == nil) then
					return;
				else
					iname = globalWindow.currentlySelected.content;
				end
			else
				iname = globalWindow.currentInstance;
			end
			
			if(globalWindow.popup ~= nil) then
				globalWindow.popup:Close();
			end
			
			local instance = globalWindow.settings.settingsTable.instances[iname];
			globalWindow.popup = globalWindow:createPopup(iname);
			globalWindow.popup.nameTextbox:SetText(iname);
			if(instance.instanceType == "raid") then
				globalWindow.popup.raidCheckbox:SetChecked(true);
			elseif(instance.instanceType == "skirmish") then
				globalWindow.popup.skirmishCheckbox:SetChecked(true);
			elseif(instance.instanceType == "fellowship") then
				globalWindow.popup.fellowshipCheckbox:SetChecked(true);
			elseif(instance.instanceType == "warband") then
				globalWindow.popup.warbandCheckbox:SetChecked(true);
			else
				globalWindow.popup.otherCheckbox:SetChecked(true);
			end
			globalWindow.popup.keywordsTextbox:SetText(globalWindow.settings:CreateKeywordsText(instance.messageParts));
			if(globalWindow.settings:ResetTimesOnDay(instance.resetDays,1)) then globalWindow.popup.sundayCheckbox:SetChecked(true); end
			if(globalWindow.settings:ResetTimesOnDay(instance.resetDays,2)) then globalWindow.popup.mondayCheckbox:SetChecked(true); end
			if(globalWindow.settings:ResetTimesOnDay(instance.resetDays,3)) then globalWindow.popup.tuesdayCheckbox:SetChecked(true); end
			if(globalWindow.settings:ResetTimesOnDay(instance.resetDays,4)) then globalWindow.popup.wednesdayCheckbox:SetChecked(true); end
			if(globalWindow.settings:ResetTimesOnDay(instance.resetDays,5)) then globalWindow.popup.thursdayCheckbox:SetChecked(true); end
			if(globalWindow.settings:ResetTimesOnDay(instance.resetDays,6)) then globalWindow.popup.fridayCheckbox:SetChecked(true); end
			if(globalWindow.settings:ResetTimesOnDay(instance.resetDays,7)) then globalWindow.popup.saturdayCheckbox:SetChecked(true); end
			
			globalWindow.popup.timeTextbox:SetText(instance.resetTime);
			
		end;
		
	self.removeButton = Turbine.UI.Lotro.Button();
	self.removeButton:SetPosition(350,465);
	self.removeButton:SetSize(130,20);
	self.removeButton:SetText("Remove Instance");
	self.removeButton:SetParent(self);
	self.removeButton.Click = 
		function(sender,args)
			local iname = nil;
			
			if(globalWindow.popup ~= nil) then
				globalWindow.popup:Close();
				globalWindow.popup = nil;
			end
			
			if(globalWindow.currentTab == "instances") then
				if(globalWindow.currentlySelected == nil) then
					return;
				else
					iname = globalWindow.currentlySelected.content;
					globalWindow:unselect();
					globalWindow.currentlySelected = nil;
				end
			else
				if(globalWindow.currentlySelected ~= nil) then
					globalWindow:unselect();
				end
				globalWindow.currentlySelected = nil;

				iname = globalWindow.currentInstance;
			end
			
			globalWindow.settings:RemoveInstance(iname);
			globalWindow.currentTab = "instances";
			if(table.getn(globalWindow.settings.settingsTable.instances) > 0) then
				globalWindow.currentInstance = 1;
			end
			globalWindow.currentPlayer = globalWindow.settings.curPlayer;
	
			globalWindow:refreshCurrentTab();
		end;
		
	self.currentTab = "instances";
	self.currentPlayer = self.settings.curPlayer;
	
	self:refreshCurrentTab();
end

function TrackerWindow:createPopup(iname)
	local popup = Turbine.UI.Lotro.Window();
	popup:SetSize(500,400);
	popup:SetText("Add Instance");
	popup:SetVisible(true);
	
	popup.nameLabel = Turbine.UI.TextBox();
	popup.nameLabel:SetPosition(20,40);
	popup.nameLabel:SetSize(50,30);
	popup.nameLabel:SetText("Name");
	popup.nameLabel:SetEnabled(false);
	popup.nameLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.nameLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	popup.nameLabel:SetParent(popup);
	
	popup.nameTextbox = Turbine.UI.Lotro.TextBox();
	popup.nameTextbox:SetPosition(80,40);
	popup.nameTextbox:SetSize(350,30);
	popup.nameTextbox:SetText("");
	popup.nameTextbox:SetParent(popup);
	
	popup.type = "raid";
	popup.checkRecursionStopper = nil;
	
	popup.raidCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.raidCheckbox:SetPosition(20,90);
	popup.raidCheckbox:SetSize(70,30);
	popup.raidCheckbox:SetChecked(true);
	popup.raidCheckbox:SetParent(popup);
	popup.raidCheckbox:SetText("");
	popup.raidCheckbox:SetEnabled(true);
	popup.raidCheckbox.CheckedChanged = 
		function(sender,args)
			if(popup.checkRecursionStopper == nil) then
			popup.checkRecursionStopper = 1;
			popup.type = "raid";
			popup.fellowshipCheckbox:SetChecked(false);
			popup.skirmishCheckbox:SetChecked(false);
			popup.otherCheckbox:SetChecked(false);
			popup.raidCheckbox:SetChecked(true);
			popup.warbandCheckbox:SetChecked(false);
			popup.checkRecursionStopper = nil;
			end
		end;

	popup.raidTextbox = Turbine.UI.TextBox();
	popup.raidTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.raidTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.raidTextbox:SetPosition(40,90);
	popup.raidTextbox:SetSize(50,30);
	popup.raidTextbox:SetText("Raid");
	popup.raidTextbox:SetParent(popup);
	
	popup.fellowshipCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.fellowshipCheckbox:SetPosition(90,90);
	popup.fellowshipCheckbox:SetSize(115,30);
	popup.fellowshipCheckbox:SetChecked(false);
	popup.fellowshipCheckbox:SetParent(popup);
	popup.fellowshipCheckbox:SetText("");
	popup.fellowshipCheckbox.CheckedChanged = 
		function(sender,args)
			if(popup.checkRecursionStopper == nil) then
			popup.type = "fellowship";
			popup.checkRecursionStopper = 1;
			popup.fellowshipCheckbox:SetChecked(true);
			popup.skirmishCheckbox:SetChecked(false);
			popup.otherCheckbox:SetChecked(false);
			popup.raidCheckbox:SetChecked(false);
			popup.warbandCheckbox:SetChecked(false);
			popup.checkRecursionStopper = nil;
			end
		end;

	popup.fellowshipTextbox = Turbine.UI.TextBox();
	popup.fellowshipTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.fellowshipTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.fellowshipTextbox:SetPosition(110,90);
	popup.fellowshipTextbox:SetSize(95,30);
	popup.fellowshipTextbox:SetText("Fellowship");
	popup.fellowshipTextbox:SetParent(popup);
	
	popup.skirmishCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.skirmishCheckbox:SetPosition(205,90);
	popup.skirmishCheckbox:SetSize(100,30);
	popup.skirmishCheckbox:SetChecked(false);
	popup.skirmishCheckbox:SetParent(popup);
	popup.skirmishCheckbox:SetText("");
	popup.skirmishCheckbox.CheckedChanged = 
		function(sender,args)
			if(popup.checkRecursionStopper == nil) then
			popup.type = "skirmish";
			popup.checkRecursionStopper = 1;
			popup.fellowshipCheckbox:SetChecked(false);
			popup.skirmishCheckbox:SetChecked(true);
			popup.otherCheckbox:SetChecked(false);
			popup.raidCheckbox:SetChecked(false);
			popup.warbandCheckbox:SetChecked(false);
			popup.checkRecursionStopper = nil;
			end
		end;
		
	popup.skirmishTextbox = Turbine.UI.TextBox();
	popup.skirmishTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.skirmishTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.skirmishTextbox:SetPosition(225,90);
	popup.skirmishTextbox:SetSize(80,30);
	popup.skirmishTextbox:SetText("Skirmish");
	popup.skirmishTextbox:SetParent(popup);

	popup.warbandCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.warbandCheckbox:SetPosition(305,90);
	popup.warbandCheckbox:SetSize(100,30);
	popup.warbandCheckbox:SetChecked(false);
	popup.warbandCheckbox:SetParent(popup);
	popup.warbandCheckbox:SetText("");
	popup.warbandCheckbox.CheckedChanged = 
		function(sender,args)
			if(popup.checkRecursionStopper == nil) then
			popup.type = "warband";
			popup.checkRecursionStopper = 1;
			popup.fellowshipCheckbox:SetChecked(false);
			popup.skirmishCheckbox:SetChecked(false);
			popup.otherCheckbox:SetChecked(false);
			popup.raidCheckbox:SetChecked(false);
			popup.warbandCheckbox:SetChecked(true);
			popup.checkRecursionStopper = nil;
			end

		end;		

	popup.warbandTextbox = Turbine.UI.TextBox();
	popup.warbandTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.warbandTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.warbandTextbox:SetPosition(325,90);
	popup.warbandTextbox:SetSize(80,30);
	popup.warbandTextbox:SetText("Warband");
	popup.warbandTextbox:SetParent(popup);
		
	popup.otherCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.otherCheckbox:SetPosition(405,90);
	popup.otherCheckbox:SetSize(150,30);
	popup.otherCheckbox:SetChecked(false);
	popup.otherCheckbox:SetParent(popup);
	popup.otherCheckbox:SetText("");
	popup.otherCheckbox.CheckedChanged = 
		function(sender,args)
			if(popup.checkRecursionStopper == nil) then
			popup.type = "other";
			popup.checkRecursionStopper = 1;
			popup.fellowshipCheckbox:SetChecked(false);
			popup.skirmishCheckbox:SetChecked(false);
			popup.otherCheckbox:SetChecked(true);
			popup.raidCheckbox:SetChecked(false);
			popup.warbandCheckbox:SetChecked(false);
			popup.checkRecursionStopper = nil;
			end

		end;		

	popup.otherTextbox = Turbine.UI.TextBox();
	popup.otherTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.otherTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.otherTextbox:SetPosition(425,90);
	popup.otherTextbox:SetSize(130,30);
	popup.otherTextbox:SetText("Other");
	popup.otherTextbox:SetParent(popup);
	
	popup.keywordsLabel = Turbine.UI.TextBox();
	popup.keywordsLabel:SetPosition(150,130);
	popup.keywordsLabel:SetSize(100,30);
	popup.keywordsLabel:SetText("Keywords");
	popup.keywordsLabel:SetEnabled(false);
	popup.keywordsLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.keywordsLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	popup.keywordsLabel:SetParent(popup);	
	
	popup.keywordsTextbox = Turbine.UI.Lotro.TextBox();
	popup.keywordsTextbox:SetPosition(20,170);
	popup.keywordsTextbox:SetSize(410,60);
	popup.keywordsTextbox:SetText("");
	popup.keywordsTextbox:SetParent(popup);
	
	popup.mondayCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.mondayCheckbox:SetPosition(20,240);
	popup.mondayCheckbox:SetSize(50,30);
	popup.mondayCheckbox:SetChecked(false);
	popup.mondayCheckbox:SetParent(popup);
	popup.mondayCheckbox:SetText("");
		
	popup.mondayTextbox = Turbine.UI.TextBox();
	popup.mondayTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.mondayTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.mondayTextbox:SetPosition(40,240);
	popup.mondayTextbox:SetSize(90,30);
	popup.mondayTextbox:SetText("Monday");
	popup.mondayTextbox:SetParent(popup);
	
	popup.tuesdayCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.tuesdayCheckbox:SetPosition(20,270);
	popup.tuesdayCheckbox:SetSize(50,30);
	popup.tuesdayCheckbox:SetChecked(false);
	popup.tuesdayCheckbox:SetParent(popup);
	popup.tuesdayCheckbox:SetText("");
		
	popup.tuesdayTextbox = Turbine.UI.TextBox();
	popup.tuesdayTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.tuesdayTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.tuesdayTextbox:SetPosition(40,270);
	popup.tuesdayTextbox:SetSize(90,30);
	popup.tuesdayTextbox:SetText("Tuesday");
	popup.tuesdayTextbox:SetParent(popup);
	
	popup.wednesdayCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.wednesdayCheckbox:SetPosition(20,300);
	popup.wednesdayCheckbox:SetSize(50,30);
	popup.wednesdayCheckbox:SetChecked(false);
	popup.wednesdayCheckbox:SetParent(popup);
	popup.wednesdayCheckbox:SetText("");
		
	popup.wednesdayTextbox = Turbine.UI.TextBox();
	popup.wednesdayTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.wednesdayTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.wednesdayTextbox:SetPosition(40,300);
	popup.wednesdayTextbox:SetSize(90,30);
	popup.wednesdayTextbox:SetText("Wednesday");
	popup.wednesdayTextbox:SetParent(popup);
	
	popup.thursdayCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.thursdayCheckbox:SetPosition(20,330);
	popup.thursdayCheckbox:SetSize(50,30);
	popup.thursdayCheckbox:SetChecked(false);
	popup.thursdayCheckbox:SetParent(popup);
	popup.thursdayCheckbox:SetText("");
		
	popup.thursdayTextbox = Turbine.UI.TextBox();
	popup.thursdayTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.thursdayTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.thursdayTextbox:SetPosition(40,330);
	popup.thursdayTextbox:SetSize(90,30);
	popup.thursdayTextbox:SetText("Thursday");
	popup.thursdayTextbox:SetParent(popup);
	
	popup.fridayCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.fridayCheckbox:SetPosition(140,255);
	popup.fridayCheckbox:SetSize(50,30);
	popup.fridayCheckbox:SetChecked(false);
	popup.fridayCheckbox:SetParent(popup);
	popup.fridayCheckbox:SetText("");
		
	popup.fridayTextbox = Turbine.UI.TextBox();
	popup.fridayTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.fridayTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.fridayTextbox:SetPosition(160,255);
	popup.fridayTextbox:SetSize(90,30);
	popup.fridayTextbox:SetText("Friday");
	popup.fridayTextbox:SetParent(popup);
	
	popup.saturdayCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.saturdayCheckbox:SetPosition(140,285);
	popup.saturdayCheckbox:SetSize(50,30);
	popup.saturdayCheckbox:SetChecked(false);
	popup.saturdayCheckbox:SetParent(popup);
	popup.saturdayCheckbox:SetText("");
		
	popup.saturdayTextbox = Turbine.UI.TextBox();
	popup.saturdayTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.saturdayTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.saturdayTextbox:SetPosition(160,285);
	popup.saturdayTextbox:SetSize(90,30);
	popup.saturdayTextbox:SetText("Saturday");
	popup.saturdayTextbox:SetParent(popup);
	
	popup.sundayCheckbox = Turbine.UI.Lotro.CheckBox();
	popup.sundayCheckbox:SetPosition(140,315);
	popup.sundayCheckbox:SetSize(50,30);
	popup.sundayCheckbox:SetChecked(false);
	popup.sundayCheckbox:SetParent(popup);
	popup.sundayCheckbox:SetText("");
		
	popup.sundayTextbox = Turbine.UI.TextBox();
	popup.sundayTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.sundayTextbox:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.sundayTextbox:SetPosition(160,315);
	popup.sundayTextbox:SetSize(90,30);
	popup.sundayTextbox:SetText("Sunday");
	popup.sundayTextbox:SetParent(popup);

	popup.timeLabel = Turbine.UI.TextBox();
	popup.timeLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18);
	popup.timeLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	popup.timeLabel:SetPosition(280,260);
	popup.timeLabel:SetSize(90,30);
	popup.timeLabel:SetText("Time");
	popup.timeLabel:SetParent(popup);
	
	popup.timeTextbox = Turbine.UI.Lotro.TextBox();
	popup.timeTextbox:SetPosition(330,260);
	popup.timeTextbox:SetSize(35,30);
	popup.timeTextbox:SetText("");
	popup.timeTextbox:SetParent(popup);
	
	popup.saveButton = Turbine.UI.Lotro.Button();
	popup.saveButton:SetPosition(280,320);
	popup.saveButton:SetSize(80,50);
	popup.saveButton:SetText("Save");
	popup.saveButton:SetParent(popup);
	popup.saveButton.Click =
		function(sender,args)
			local name = popup.nameTextbox:GetText();
			local messageParts = globalWindow.settings:ParseMessageParts(popup.keywordsTextbox:GetText());
			local instanceType = popup.type;
			local resetDays = globalWindow.settings:ParseResetDays(
				popup.mondayCheckbox:IsChecked(),
				popup.tuesdayCheckbox:IsChecked(),
				popup.wednesdayCheckbox:IsChecked(),
				popup.thursdayCheckbox:IsChecked(),
				popup.fridayCheckbox:IsChecked(),
				popup.saturdayCheckbox:IsChecked(),
				popup.sundayCheckbox:IsChecked()
			);

			local resetTime = tonumber(popup.timeTextbox:GetText());
			if(iname ~= nil) then
				local instance = globalWindow.settings.settingsTable.instances[iname];
				instance.name = name;
				instance.messageParts = messageParts;
				instance.instanceType = instanceType;
				instance.resetDays = resetDays;
				instance.resetTime = resetTime;
			else
				globalWindow.settings:AddInstance(name,messageParts,instanceType,resetDays,resetTime);
			end

			popup:Close();
			globalWindow.popup = nil;
			globalWindow.settings:Save();
			globalWindow:refreshCurrentTab();

		end;
	
	return popup;
end

function TrackerWindow:switchToPlayersTab(iname)
	self.currentTab = "players";
	self.currentInstance = iname;
	self:refreshCurrentTab();
end

function TrackerWindow:switchToInstancesTab(pnum)
	self.currentTab = "instances";
	self.currentPlayer = pnum;
	self:refreshCurrentTab();
end

function TrackerWindow:refreshCurrentTab()
	self.settings:PerformResets();
	if(self.currentTab == "instances") then self:displayInstancesTab();
	elseif(self.currentTab == "players") then self:displayPlayersTab();
	elseif(self.currentTab == "settings") then self:displaySettingsTab(); end
end

function TrackerWindow:displayPlayersTab()
	self.currentTab = "players";

	if(self.playerSelectorWindow ~= nil) then self.playerSelectorWindow:SetVisible(false); end
	if(self.playerSelectorWindowScroll ~= nil) then self.playerSelectorWindowScroll:SetVisible(false); end
	if(self.instanceSelectorWindow ~= nil) then self.instanceSelectorWindow:SetVisible(false); end
	if(self.instanceSelectorWindowScroll ~= nil) then self.instanceSelectorWindowScroll:SetVisible(false); end
	
	self.playerSelectorWindow = Turbine.UI.TreeView();
	self.playerSelectorWindow:SetPosition(20,20);
	self.playerSelectorWindow:SetSize(450,335);
	self.playerSelectorWindow:SetVisible(true);
	
	self.playerSelectorWindowScroll = Turbine.UI.Lotro.ScrollBar();
	self.playerSelectorWindowScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.playerSelectorWindowScroll:SetPosition(470,20);
	self.playerSelectorWindowScroll:SetSize(10,335);
	self.playerSelectorWindow:SetVerticalScrollBar(self.playerSelectorWindowScroll);
	
	local nodeList = self.playerSelectorWindow:GetNodes();
	
	for k,player in pairs(self.settings.settingsTable.players) do
		nodeList:Add(CTTreeNodeItem(player));
	end
	
	self.playerSelectorWindow:SetParent(self.mainWindow);
	self.playerSelectorWindowScroll:SetParent(self.mainWindow);

	globalWindow.contextTextBox:SetText(self.currentInstance);
	self:updatePlayerTabForInstance();
end

function TrackerWindow:displayInstancesTab()
	self.currentTab = "instances";
	
	if(self.playerSelectorWindow ~= nil) then self.playerSelectorWindow:SetVisible(false); end
	if(self.playerSelectorWindowScroll ~= nil) then self.playerSelectorWindowScroll:SetVisible(false); end
	if(self.instanceSelectorWindow ~= nil) then self.instanceSelectorWindow:SetVisible(false); end
	if(self.instanceSelectorWindowScroll ~= nil) then self.instanceSelectorWindowScroll:SetVisible(false); end

	self.instanceSelectorWindow = Turbine.UI.TreeView();
	self.instanceSelectorWindow:SetPosition(20,20);
	self.instanceSelectorWindow:SetSize(450,335);
	
	self.instanceSelectorWindowScroll = Turbine.UI.Lotro.ScrollBar();
	self.instanceSelectorWindowScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.instanceSelectorWindowScroll:SetPosition(470,20);
	self.instanceSelectorWindowScroll:SetSize(10,335);
	self.instanceSelectorWindow:SetVerticalScrollBar(self.instanceSelectorWindowScroll);
	
	local nodeList = self.instanceSelectorWindow:GetNodes();
	
	self.headerMap = {};
	
	local node = CTTreeNodeHeader("Raids");
	self.headerMap["raid"] = node;
	nodeList:Add(node);

	local node = CTTreeNodeHeader("Skirmishes");
	self.headerMap["skirmish"] = node;
	nodeList:Add(node);

	local node = CTTreeNodeHeader("Fellowship");
	self.headerMap["fellowship"] = node;
	nodeList:Add(node);
	
	local node = CTTreeNodeHeader("Warband");
	self.headerMap["warband"] = node;
	nodeList:Add(node);
	
	local node = CTTreeNodeHeader("Other");
	self.headerMap["other"] = node;
	nodeList:Add(node);

	for k,instance in pairs(self.settings.settingsTable.instances) do
		self.headerMap[instance.instanceType]:GetChildNodes():Add(CTTreeNodeItem(instance.name));
	end
	
	self.instanceSelectorWindow:SetParent(self.mainWindow);
	self.instanceSelectorWindowScroll:SetParent(self.mainWindow);

	globalWindow.contextTextBox:SetText(self.currentPlayer);
	
	self:updateInstanceTabForPlayer();
end

function TrackerWindow:updateInstanceTabForPlayer()
	local instanceTable = self.settings.settingsTable.instances;
	for q,header in pairs(self.headerMap) do
		local nodes = header:GetChildNodes();
		local count1 = 0;
		local count2 = 0;
		
		for i=1,nodes:GetCount() do
			local node = nodes:Get(i);
			
			local instance = instanceTable[node.content];
			local priority = instance.priority[self.currentPlayer];
			
			if(priority=="important") then
				node:markAsImportant();
			elseif(priority=="standard") then
				node:markAsStandard();
			else
				node:markAsIgnore();
			end
			
			if(instanceTable[node.content].completion[self.currentPlayer] ~= nil) then
				node:setUnavailable(self.settings:GetClosestTime(instance.resetDays,instance.resetTime));
			else
				node:setAvailable();
				if(priority=="important") then
					count1 = count1 + 1;
				elseif(priority=="standard") then
					count2 = count2 + 1;
				else
					--ignore
				end
			end
		end
		
		header:setCounts(count1,count2);
	end
end

function TrackerWindow:updatePlayerTabForInstance()
	local playerTable = self.settings.settingsTable.players;
	local instanceTable = self.settings.settingsTable.instances;
	local nodes = self.playerSelectorWindow:GetNodes();
	
	for i=1,nodes:GetCount() do
		local node = nodes:Get(i);
		
		local instance = instanceTable[self.currentInstance];
		local priority = instance.priority[node.content];
		
		if(priority=="important") then
			node:markAsImportant();
		elseif(priority=="standard") then
			node:markAsStandard();
		else
			node:markAsIgnore();
		end
		
		if(instance.completion[node.content] ~= nil) then
			node:setUnavailable(self.settings:GetClosestTime(instance.resetDays,instance.resetTime));
		else
			node:setAvailable();
		end
	end
end